#!/bin/bash
#

# buildRoot.sh <product_directory> <e5|nu:e5|i1|nu:i1> <debug|opt|prof> [tar] [source]

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e5|nu:e5|i1|nu:i1> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
    qlist=`echo ${fullqual} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`
    qualver=`echo ${qlist} | tr " " "_"`

    if [ "${basequal}" = "e5" ]
    then
	shortqual=e
	cqual=e5
        xcc=gcc
    elif [ "${basequal}" = "nu:e5" ]
    then
	shortqual=nu
	cqual=e5
        xcc=gcc
    elif [ "${basequal}" = "i1" ]
    then
	shortqual=i
	cqual=i1
        xcc=icc
        (( useicc = 1 ))
    elif [ "${basequal}" = "nu:i1" ]
    then
	shortqual=nu
	cqual=i1
        xcc=icc
        (( useicc = 1 ))
    else
       echo "ERROR: unrecognized qualifier"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
      (( darwin = 1 ))
      flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=root
origpkgver=v5_34_18
pkgver=${origpkgver}b
ssibuildshims_version=v0_07_04
pkgtarver=`echo ${origpkgver} | sed -e 's/_/./g'`
pkgtarfile=root_${pkgtarver}.source.tar.gz

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# use ROOTSYS/MyRules.mk
if [ "${extraqual}" = "opt" ]
then
    rules=MyRules_opt.mk
elif [ "${extraqual}" = "debug" ]
then
    extracommand="--build=debug"
    rules=MyRules_debug.mk
elif [ "${extraqual}" = "prof" ]
then
    rules=MyRules_prof.mk
fi

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup root
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed"

if [ -z ${FFTW_INC} ]
then
   echo "ERROR: failed to setup fftw"
   exit 1
else
   echo "FFTW_INC: ${FFTW_INC}"
fi

if [ -z ${LIBXML2_INC} ]
then
   echo "ERROR: failed to setup libxml"
   exit 1
else
   echo "LIBXML2_INC: ${LIBXML2_INC}"
fi

if [ -z ${XROOTD_INC} ]
then
echo "ERROR: failed to setup xrootd"
exit 1
else
echo "XROOTD_INC: ${XROOTD_INC}"
fi

if [ "${shortqual}" = "nu" ]
then
    if [ -z ${POSTGRESQL_VERSION} ]
    then
       echo "ERROR: failed to setup postgresql"
       exit 1
    else
       echo "postgresql version: ${POSTGRESQL_VERSION}"
    fi

    if [ -z ${MYSQL_CLIENT_VERSION} ]
    then
       echo "ERROR: failed to setup mysql_client"
       exit 1
    else
       echo "mysql_client version: ${MYSQL_CLIENT_VERSION}"
    fi
fi

if [ -z ${PYTHONHOME} ]
then
   echo "ERROR: PYTHONHOME is undefined"
   echo "       something is very wrong"
   exit 1
fi

cd ${sourcedir} && rm -rf root
tar xzf ${tardir}/${pkgtarfile}

# get the extra rules 
if [ -n ${rules} ]
then
   cp ${patchdir}/${rules} ${blddir}/MyRules.mk
fi
if [[ "${cqual}" == e[45] ]]
then
  c11="--enable-c++11"
  echo "CXXFLAGS += -std=c++11 -Wno-deprecated-declarations" >> ${blddir}/MyRules.mk
  echo "NOOPT += -std=c++11 -Wno-deprecated-declarations" >> ${blddir}/MyRules.mk
  compiler_command="--with-cc=gcc --with-cxx=g++ --with-ld=g++"
elif [ "${cqual}" = "i1" ]
then
  c11="--enable-c++11"
  echo "CXXFLAGS += -std=c++11" >> ${blddir}/MyRules.mk
  echo "NOOPT += -std=c++11" >> ${blddir}/MyRules.mk
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${sourcedir}/root

# apply Fermi patches
patch="${patchdir}/fermi_root_5_34.patch"
patch -b -p0 < "${patch}" || die "ERROR: application of patch ${patch} failed."
patch="${patchdir}/fix-DropBasket.patch"
patch -b -p1 < "${patch}" || die "ERROR: application of patch ${patch} failed."
cd ${blddir}


if [ ${OS1} = "Darwin" ]
then
  darwin_xopts=(--disable-cocoa --enable-builtin-afterimage \
                --enable-builtin-ftgl --enable-builtin-freetype \
                --enable-builtin-glew --enable-builtin-pcre \
                --enable-builtin-zlib --enable-builtin-lzma)
  makearch="macosx64${useicc:+icc}"
  # Remove /usr/local from search paths.
  perl -wapi\.bak -e 'm&prefix& or (s&/usr/local[^\s;"'"'"'\)]*&&g);' ${sourcedir}/root/configure
else
  makearch="linuxx8664${xcc}"
fi

if [ "${shortqual}" = "nu" ]
then
${sourcedir}/root/configure ${makearch} ${extracommand} ${compiler_command} \
--enable-gdml ${c11} \
--enable-roofit \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--with-xrootd-incdir=${XROOTD_INC}/xrootd \
--with-xrootd-libdir=${XROOTDROOT}/lib/ \
--enable-minuit2 \
--disable-xft \
--with-gsl-incdir=${GSL_INC} \
--with-gsl-libdir=${GSL_LIB} \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib \
--with-pythia6-libdir=${PYLIB} \
--with-pgsql-incdir=${POSTGRESQL_FQ_DIR}/include \
--with-pgsql-libdir=${POSTGRESQL_FQ_DIR}/lib \
--with-mysql-incdir=${MYSQL_CLIENT_FQ_DIR}/include \
--with-mysql-libdir=${MYSQL_CLIENT_FQ_DIR}/lib \
--enable-mathmore \
--enable-gsl-shared \
${darwin:+"${darwin_xopts[@]}"}
else # Vanilla now has minuit2
${sourcedir}/root/configure ${makearch} ${extracommand}  ${compiler_command} \
--enable-gdml ${c11} \
--enable-roofit \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--disable-mysql \
--with-xrootd-incdir=${XROOTD_INC}/xrootd \
--with-xrootd-libdir=${XROOTDROOT}/lib/ \
--enable-minuit2 \
--disable-xft \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib \
${darwin:+"${darwin_xopts[@]}"}
fi

(( $? == 0 )) || die "ERROR: configure failed."

make -j ${ncore} || die "ERROR: make failed."

# now install in ROOTSYS
make install || die "ERROR: make install failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# need to fix paths:
# bin/root-config
# lib64/Geant4-9.5.2/Geant4*.cmake
# share/Geant4-9.5.2/geant4make/geant4make*sh

fixfile=/tmp/root-fix-${qualdir}-$$$$.sh.sh
rm -f $fixfile

echo "#!/bin/bash" > $fixfile
##echo " set -x " >>  $fixfile
echo "mv ${pkgdir}/bin/root-config ${pkgdir}/bin/root-config.bak" >> $fixfile
echo "cat ${pkgdir}/bin/root-config.bak \\" >> $fixfile
echo "     | sed 's%${LIBXML2_FQ_DIR}%\${LIBXML2_FQ_DIR}%g' \\" >> $fixfile
echo "     | sed 's%${FFTW_FQ_DIR}%\${FFTW_FQ_DIR}%g' \\" >> $fixfile
echo "     | sed 's%${XROOTDROOT}%\${XROOTDROOT}%g' \\" >> $fixfile
if [ "${shortqual}" = "nu" ]
then
echo "     | sed 's%${PYLIB}%\${PYLIB}%g' \\" >> $fixfile
echo "     | sed 's%${POSTGRESQL_FQ_DIR}%\${POSTGRESQL_FQ_DIR}%g' \\" >> $fixfile
echo "     | sed 's%${MYSQL_CLIENT_FQ_DIR}%\${MYSQL_CLIENT_FQ_DIR}%g' \\" >> $fixfile
fi
echo "     > ${pkgdir}/bin/root-config" >> $fixfile
echo "chmod +x  ${pkgdir}/bin/root-config" >> $fixfile
chmod +x $fixfile
$fixfile

rm -f $fixfile

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || die "ERROR: setup failed"
echo "${package} is installed at ${ROOTSYS}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
